#!/bin/bash
# File  : 01_example_readme_1
# Author: Felix Kuehnl
# Date  : 2017-10-23

# Enable 'unofficial strict mode'.
#   -e: exit immediately if any command has non-zero return value. This can be
#       escaped by appending '|| true' to a command, or use
#       'set +e; cmd; set -e'
#   -u: exit immediately when an unset variable is dereferenced
#   -o pipefail: pipelines return the error code of the rightmost command that
#       failed (instead of always returning the value of the rightmost
#       command). Result with -e: script exits if any subcommand within a pipe
#       fails.
set -euo pipefail

# Abort on errors, displaying error message + code, kill all running jobs.
clean_and_die() {
    error_code="$1"; error_message="$2"
    echo -e "\nERROR: $error_message ($error_code) in script" \
        "'$(basename $0)'" 1>&2
    jobs=$(jobs -pr); [ -z "$jobs" ] || kill $(jobs -pr)
    exit $error_code
}

trap 'clean_and_die $? "terminated unexpectedly at line $LINENO"' ERR
trap 'clean_and_die $? "interrupted"'           INT
trap 'clean_and_die $? "caught TERM signal"'    TERM


##############################################################################
##                              Script options                              ##
##############################################################################

# Name and version of this script
SCRIPT_NAME='01_example_readme_1/test.sh'
SCRIPT_VERSION='v0.1'

# Number of positional args. Set negative to disable check.
NO_POSITIONAL_ARGS=0

# Set options to parse here. Colon (:) after letter means option has a value.
OPT_STRING='vh'

HEADING=$( perl -e '        # Center string by adding padding spaces
    $line_length=78; $s="* * * @ARGV * * *"; $pad=($line_length-length $s)/2;
    $pad=0 if $pad<0; print " " x $pad, $s
' $SCRIPT_NAME $SCRIPT_VERSION )
usage=$( cat <<EndOfUsage
$HEADING

Test whether the first example from the readme file runs correctly.

Usage:  $SCRIPT_NAME [ARGS]

Arguments:  [...] denotes default values, xx doubles, ii ints, ss strings
    -v:     Be verbose and show debug messages.
    -h:     Display this help and exit.
EndOfUsage
)
unset HEADING


##############################################################################
##                              Core functions                              ##
##############################################################################

# Display passed error message an exit with error code 1
die() {
    echo "ERROR: $*" 1>&2; exit 1
}

# # Check whether global error flag $? is set and if so, die with the
# # passed error message
# assert() {
#     [ $? -ne 0 ] && die "$@"
# }

# Ensure existence and non-emptiness of a (regular) file
# Arguments:
#   file_name, and optionally
# Optional arguments:
#   file_description: To be displayed in error message [File]
#   can_be_empty: Whether empty files are allowed [false]
assert_file_exists() {
    local file_name="$1"
    local file_description="${2-File}"
    local can_be_empty=${3:-}
    local error=
    local message=

    [ -f "$file_name" ] || { error=1; message='not found'; }
    [ ! -n "$error" ] && [ ! -n "$can_be_empty" ] && [ ! -s "$file_name" ] &&
        { error=1; message='is empty'; }

    [ -z "$error" ] ||
        die "$file_description '$file_name' $message"
}

# Print a message if global variable is set, usually via -v switch
dbgm() {
    if [ -n "${BE_VERBOSE-}" ]; then
        echo "$@" 1>&2
    fi
    return 0
}

# Indent all text piped through with n spaces.
# Argument: n -- amount of spaces to indent [4]
indent() {
    local indent=${1-4}     # indent 4 spaces by default
    perl -pe "print q{ } x $indent"
}


##############################################################################
##                              Default values                              ##
##############################################################################

BE_VERBOSE=${BE_VERBOSE:+1}
n=$'\n'                 # Newline
t=$'\t'                 # Tab
export LC_COLLATE='C'   # set sys language to C to avoid problems with sorting
export LC_NUMERIC='C'   # also recognize '.' as decimal point


##############################################################################
##                              Option parsing                              ##
##############################################################################

while getopts "$OPT_STRING" opt; do
    case $opt in
#       e)
#           example=$OPTARG ;;
        v)
            BE_VERBOSE=1 ;;
        h)
            echo "$usage"
            exit 0
            ;;
        *)
            die "Invalid option, use -h to get help."
    esac
done

#### Positional params ####
shift $(($OPTIND-1))       # shift positional parameters to position 1, 2, ...
wrongNoParamMsg='Wrong number of positional parameters, use -h to get help'
[ $# -eq $NO_POSITIONAL_ARGS ] || [ $NO_POSITIONAL_ARGS -lt 0 ] ||
    die "$wrongNoParamMsg"


#### Collect positional args ####
# Use array/"${var[@]}" notation to preserve file names containing whitespace
# input_files=("$@")


#### Print debug output like e. g. passed arguments ####
dbgm "                      * * * $SCRIPT_NAME $SCRIPT_VERSION  * * *"


##############################################################################
##                           Function definitions                           ##
##############################################################################

# Function to add decimal marks / thousands separators to an integer
_1000s_sep() {
    _1000S_SEP=$(
    perl -pne '
            while ( s/([-+]?\d+)(\d{3})/\1,\2/g ) {}
        ' <<< "$1"
    )
}


##############################################################################
##                                   Main                                   ##
##############################################################################


# Copy example to a new directory
tmp_dir="$( mktemp -d 'tmp_dir_XXX' )"
[ -d "$tmp_dir" ] || die 'Failed to create temp dir'
cp '../../example/RS1' "./$tmp_dir"
cd "$tmp_dir"

kinSimRiboswitch \
    -s'(((((...((((((((.....)))))...)))...))))).............................'\
    -E 18 -L 1e-3 RS1


cd ..
rm -rf "$tmp_dir"

exit 0
# EOF

